/*
 * Compile me with:
 *   gcc -o tut tut.c $(pkg-config --cflags --libs gtk+-2.0 gmodule-2.0)
*/
#include <gtk/gtk.h>

#include "support.h"
#include "max9526-i2c.h"

#define UI_FILE "/usr/share/max9526-gtk/max9526.glade"
//#define UI_FILE "max9526.glade"

static unsigned is_initialized = FALSE;
/*
REGISTER		B7 	B6 	B5 	B4 	B3 	B2 	B1 	B0 	REG 	POWER-
											ADDR	ON
											STATE
Status 0 		VID1 	VID2 	0 	CTHR 	ADCOVR 	HLOCK 	NONSTD 	LSTLCK 	0x00 	n/a
Status 1 		0 	L525 	0 	0 	0 	0 	0 	ACP 	0x01 	n/a
IRQ MASK 0 		IVID1 	IVID2 	0 	ICTHR 	IADCOVR	IHLOCK	INONSTD	ILSTLCK	0x02	0x00
IRQ MASK 1 		0 	IL525 	0	0 	0 	0 	0 	IACP 	0x03 	0x00
Standard Select,		STDSEL		AUTOD 	SHDN 	RESET 	SLEEP 	RESET_S	0x04 	0x10
Shutdown, and
Control
Contrast					CONT 					0x05 	0x80
Brightness					BRIGHT 					0x06 	0x00
Hue 						HUE 					0x07 	0x80
Saturation 					SATU 					0x08 	0x88
Video Input Select	AUTOSEL	INSEL 	DCRESTORE_RANGE	0	0	D_CLMP_	0	0x09 	0x02
and Clamp 								DIS
Gain Control 		CRAGC 	CMPAGC 	0 	ADAGC 		ADCGAIN			0x0A 	0x00
Color Kill		BW 	CRKDIS 	1 	0 		CTHRSH 			0x0B 	0x23
Output Test Signal 	RAWADC 	0 	TGEnab 	TGTIM 	TGSRC 	0 	CBAR		0x0C 	0x00
Clock and Output	0 	CLIP 	LLC_INV SEL_54 	XTAL_	HSVS 	DATAZ	LLCZ	0x0D 	0x00
						MHZ 	DIS
PLL Control 		0 	0 	LLC_MODE	PLLBYP		PLLBW		0x0E 	0x03
Miscellaneous 		0 	0 	DISA	1		SSLICE			0x0F 	0x18
					AFLT
*/                                
#define MAX9526_STATUS0       0x00
#define MAX9526_STATUS1       0x01
#define MAX9526_IRQ_MASK0     0x02
#define MAX9526_IRQ_MASK1     0x03
#define MAX9526_STD_SELECT    0x04
#define MAX9526_CONTRAST      0x05
#define MAX9526_BRIGHTNESS    0x06
#define MAX9526_HUE           0x07
#define MAX9526_SATURATION    0x08
#define MAX9526_VIDEO_INPUT   0x09
#define MAX9526_GAIN_CTRL     0x0A
#define MAX9526_COLOR_KILL    0x0B
#define MAX9526_TEST_SIGNAL   0x0C
#define MAX9526_CLOCK         0x0D
#define MAX9526_PLL_CTRL      0x0E
#define MAX9526_MISCELLANEOUS 0x0F 


ChData     *data;

void drawStatus0 (void);
void drawStatus1 (void);
void drawIRQ (void);
void drawVideoStandard(void);
void drawSliders (void);
void drawVideoInputSelect(void);
void drawGain(void);
void drawKill(void);
void drawTest(void);
void drawClock(void);
void drawPLL(void);
void drawMisc(void);

void refresh ( void )
{
  is_initialized = FALSE;
  drawStatus0 ();
  drawStatus1 ();
  drawIRQ ();
  drawVideoStandard();
  drawSliders ();
  drawVideoInputSelect();
  drawGain();
  drawKill();
  drawTest();
  drawClock();
  drawPLL();
  drawMisc();
  is_initialized = TRUE;
}

G_MODULE_EXPORT void ev_Refresh ( GtkWidget *widget, gpointer callback_data )
{
  refresh ();
}

G_MODULE_EXPORT void ev_IRQMask ( GtkWidget *widget, gpointer callback_data )
{
	unsigned reg;
  reg =  (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IVID1) )) << 7;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IVID2) )) << 6;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ICTHR) )) << 4;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IADCOVR) )) << 3;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IHLOCK) )) << 2;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_INONSTD) )) << 1;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ILSTLCK) ));
  write_reg_max9526(MAX9526_IRQ_MASK0, reg);
  reg =  (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IL525) )) << 6;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_IACP) )) ;
  write_reg_max9526(MAX9526_IRQ_MASK1, reg);
  refresh ();
}

G_MODULE_EXPORT void ev_SystemReset( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = 0x04;
  write_reg_max9526(MAX9526_STD_SELECT, reg);
  refresh ();
  }
}
G_MODULE_EXPORT void ev_SoftReset( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = 0x01 | read_reg_max9526(MAX9526_STD_SELECT);
  write_reg_max9526(MAX9526_STD_SELECT, reg);
  refresh ();
  }
}
G_MODULE_EXPORT void ev_VideoStandard( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_VideoStandard) ) << 5) & 0xe0;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_AUTOD) )) << 4;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_SHDN) )) << 3;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_SLEEP) )) << 1;
  write_reg_max9526(MAX9526_STD_SELECT, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_Contrast( GtkWidget *widget, gpointer callback_data )
{
  float value;
  unsigned reg;
  char buf[20];
  if(is_initialized)
  {
  value = gtk_range_get_value ( (GtkRange*) widget);
  reg = value;
  write_reg_max9526(MAX9526_CONTRAST, reg);
  
  sprintf(buf, "Luma Gain %.2f", value / 128.0);
  gtk_label_set_text ( (GtkLabel *)data->lb_LumaGain, buf );
  }
}
G_MODULE_EXPORT void ev_Brightness( GtkWidget *widget, gpointer callback_data )
{
  float value;
  signed reg;
  char buf[20];
  if(is_initialized)
  {
  value = gtk_range_get_value ( (GtkRange*) widget) ;
  reg = value;
  reg &= 0xff;
  write_reg_max9526(MAX9526_BRIGHTNESS, reg);
 
  sprintf(buf, "%.2f IRE", value * 76.22 / 128.0);
  gtk_label_set_text ( (GtkLabel *)data->lb_IRE, buf );
  }
}
G_MODULE_EXPORT void ev_Hue( GtkWidget *widget, gpointer callback_data )
{
  float value;
  unsigned reg;
  char buf[20];
  if(is_initialized)
  {
  value = gtk_range_get_value ( (GtkRange*) widget);
  reg = value;
  write_reg_max9526(MAX9526_HUE, reg);
  
  sprintf(buf, "%.1f degrees", value * 45.0 / 128.0 - 45.0);
  gtk_label_set_text ( (GtkLabel *)data->lb_Degrees, buf );
  }
}
G_MODULE_EXPORT void ev_Saturation( GtkWidget *widget, gpointer callback_data )
{
  float value;
  unsigned reg;
  char buf[20];
  if(is_initialized)
  {
  value = gtk_range_get_value ( (GtkRange*) widget);
  reg = value;
  write_reg_max9526(MAX9526_SATURATION, reg);
  
  sprintf(buf, "Chroma Gain %.2f", value/ 128.0);
  gtk_label_set_text ( (GtkLabel *)data->lb_ChromaGain, buf );
  }
}

G_MODULE_EXPORT void ev_VideoInput( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_VideoInputSrc) ) << 6) & 0xc0;
  reg |= (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_DCRestore) ) << 4) & 0x30;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_DigitalClampDis) ) > 0 ) << 1;
  write_reg_max9526(MAX9526_VIDEO_INPUT, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_Gain( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_GainADC) ) ) & 0x0f;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_GainChAGC) ) > 0 ) << 7;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_GainCoAGC) ) > 0 ) << 6;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_GainADCAGC) ) > 0 ) << 4;

  write_reg_max9526(MAX9526_GAIN_CTRL, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_Kill( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_KillThreshold) ) ) & 0x0f;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_KillBW) ) > 0 ) << 7;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_KillColour) ) > 0 ) << 6;
  reg |= 1 << 5;

  write_reg_max9526(MAX9526_COLOR_KILL, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_Test( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  	reg =  (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_TestBars) ) ) & 0x03;
  	reg |= (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_TestTiming) ) << 3  ) & 0x18;
  	reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_TestADC) ) > 0 ) << 7;
    reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_TestPattern) ) > 0 ) << 5;

    write_reg_max9526(MAX9526_TEST_SIGNAL, reg);
    refresh ();
  }
}

G_MODULE_EXPORT void ev_Clock( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_ClockClip) ) << 6 ) & 0x40;
	reg |= (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_ClockOsz) ) << 3  ) & 0x18;
	reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ClockInvLLC) ) > 0 ) << 5;
	reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ClockSync) ) > 0 ) << 2;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ClockDisableDx) ) > 0 ) << 1;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_ClockDisableLLC) ) > 0 ) ;

  write_reg_max9526(MAX9526_CLOCK, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_PLL( GtkWidget *widget, gpointer callback_data )
{
  unsigned reg;
  if(is_initialized)
  {
  reg = (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_PLLClock) ) << 4) & 0x30;
  reg |= (gtk_combo_box_get_active( GTK_COMBO_BOX(data->cob_PLLTracking) ) ) & 0x7;
  reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_PLL) ) > 0 ) << 3;
  write_reg_max9526(MAX9526_PLL_CTRL, reg);
  refresh ();
  }
}

G_MODULE_EXPORT void ev_Misc( GtkWidget *widget, gpointer callback_data )
{
  float value;
  unsigned reg;
  if(is_initialized)
  {
    value = gtk_range_get_value ( (GtkRange*)data->vs_MiscSSLICE );
    reg = value;
    reg &= 0xf;
    reg |= (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(data->cb_MiscAGC) ) > 0 ) << 5;
    reg |= 1 << 4;

    write_reg_max9526(MAX9526_MISCELLANEOUS, reg);
    refresh ();
  }
}

void drawStatus0 (void)
{
  unsigned status0;

  status0 = read_reg_max9526(MAX9526_STATUS0);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_VID1, status0 & 0x80);
  gtk_widget_set_sensitive(data->cb_VID1, status0 & 0x80); 
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_VID2, status0 & 0x40);
  gtk_widget_set_sensitive(data->cb_VID2, status0 & 0x40);  
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_CTHR, status0 & 0x10);
  gtk_widget_set_sensitive(data->cb_CTHR, status0 & 0x10);  
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ADCOVR, status0 & 0x08);
  gtk_widget_set_sensitive(data->cb_ADCOVR, status0 & 0x08); 
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_HLOCK, status0 & 0x04);
  gtk_widget_set_sensitive(data->cb_HLOCK, status0 & 0x04);  
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_NONSTD, status0 & 0x02);
  gtk_widget_set_sensitive(data->cb_NONSTD, status0 & 0x02); 
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_LSTLCK, status0 & 0x01);
  gtk_widget_set_sensitive(data->cb_LSTLCK, status0 & 0x01);  
}

void drawStatus1 (void)
{
  unsigned status1;

  status1 = read_reg_max9526(MAX9526_STATUS1);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_L525, status1 & 0x40);
  gtk_widget_set_sensitive(data->cb_L525, status1 & 0x40);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ACP, status1 & 0x01);
  gtk_widget_set_sensitive(data->cb_ACP, status1 & 0x01);  
}

void drawIRQ (void)
{
  unsigned reg;

  reg = read_reg_max9526(MAX9526_IRQ_MASK0);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IVID1, reg & 0x80);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IVID2, reg & 0x40);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ICTHR, reg & 0x10);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IADCOVR, reg & 0x08);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IHLOCK, reg & 0x04);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_INONSTD, reg & 0x02);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ILSTLCK, reg & 0x01);
  reg = read_reg_max9526(MAX9526_IRQ_MASK1);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IL525, reg & 0x40);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_IACP, reg & 0x01);
}

void drawVideoStandard(void)
{
  unsigned reg_now;

  reg_now = read_reg_max9526(MAX9526_STD_SELECT);
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_VideoStandard), reg_now >> 5);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_AUTOD, reg_now & 0x10);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_SHDN, reg_now & 0x08);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_SLEEP, reg_now & 0x02);
}

void setupVideoStandard(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;
  
  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"PAL B/G/H/I/D (standard PAL)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"PAL Combination N",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"NTSC M (standard NTSC)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"PAL M",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"N/A",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"NTSC 4.43",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"NTSC J",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"PAL60",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_VideoStandard), GTK_TREE_MODEL(store));

  cell = gtk_cell_renderer_text_new();
  
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_VideoStandard), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_VideoStandard ), cell, "text", 0, NULL ); 

  drawVideoStandard();
}

void drawSliders (void)
{
  unsigned reg;

  reg = read_reg_max9526(MAX9526_CONTRAST);
  gtk_range_set_value ( (GtkRange*) data->vs_Contrast, reg );

  reg = read_reg_max9526(MAX9526_BRIGHTNESS);
  gtk_range_set_value ( (GtkRange*) data->vs_Brightness, reg );

  reg = read_reg_max9526(MAX9526_HUE);
  gtk_range_set_value ( (GtkRange*) data->vs_Hue, reg );

  reg = read_reg_max9526(MAX9526_SATURATION);
  gtk_range_set_value ( (GtkRange*) data->vs_Saturation, reg );
}
void setupSliders (void)
{
  GtkAdjustment *adjustment; 
  adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, 0.0, 255.0, 1.0, 16.0, 0.0); 
  gtk_range_set_adjustment( (GtkRange*)data->vs_Contrast, adjustment );
  gtk_range_set_inverted ((GtkRange*) data->vs_Contrast, TRUE);

  adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, -128.0, 127.0, 1.0, 16.0, 0.0); 
  gtk_range_set_adjustment( (GtkRange*)data->vs_Brightness, adjustment );
  gtk_range_set_inverted ((GtkRange*) data->vs_Brightness, TRUE);

  adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, 0.0, 255.0, 1.0, 16.0, 0.0); 
  gtk_range_set_adjustment( (GtkRange*)data->vs_Hue, adjustment );
  gtk_range_set_inverted ((GtkRange*) data->vs_Hue, TRUE);

  adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, 0.0, 255.0, 1.0, 16.0, 0.0); 
  gtk_range_set_adjustment( (GtkRange*)data->vs_Saturation, adjustment );
  gtk_range_set_inverted ((GtkRange*) data->vs_Saturation, TRUE);
}

void drawVideoInputSelect(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_VIDEO_INPUT);
  reg = reg_now >> 6;
  if (reg > 2)
    reg = 1;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_VideoInputSrc), reg);

  reg = (reg_now >> 4) & 0x3;
  gtk_combo_box_set_active(GTK_COMBO_BOX( data->cob_DCRestore), reg);
  
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_DigitalClampDis, reg_now & 0x02);
}

void setupVideoInputSelect(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;
  
  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Video In 1",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Video In 2",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Video In Auto",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_VideoInputSrc), GTK_TREE_MODEL(store));

  
  cell = gtk_cell_renderer_text_new();
  
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_VideoInputSrc), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_VideoInputSrc ), cell, "text", 0, NULL ); 
  
  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"MediumFast (default)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Fast",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Slow",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"Medium",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX( data->cob_DCRestore), GTK_TREE_MODEL(store));

  
  cell = gtk_cell_renderer_text_new();
  
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT( data->cob_DCRestore), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT(  data->cob_DCRestore ), cell, "text", 0, NULL ); 
  
  drawVideoInputSelect();
}

void drawGain(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_GAIN_CTRL);
  reg = reg_now & 0xf;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_GainADC), reg);

  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_GainChAGC, reg_now & 0x80);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_GainCoAGC, reg_now & 0x40);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_GainADCAGC, reg_now & 0x10);
}

void setupGain(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0000 (default) 752mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0001 683 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0010 626 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0011 578 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0100 535 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0101 500 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0110 469 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0111 441 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1000 417 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1001 394 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1010 375 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1011 357 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1100 341 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1101 326 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1110 313 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1111 300 mV",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_GainADC), GTK_TREE_MODEL(store));


  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_GainADC), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_GainADC ), cell, "text", 0, NULL );

  drawGain();
}

void drawKill(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_COLOR_KILL);
  reg = reg_now & 0xf;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_KillThreshold), reg);

  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_KillBW, reg_now & 0x80);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_KillColour, reg_now & 0x40);
}

void setupKill(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0000 OFF",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0001 OFF",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0010 19 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0011 (default) 25 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0100 27 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0101 29 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0110 30 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0111 31 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1000 35 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1001 39 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1010 40 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1011 41 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1100 43 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1101 45 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1110 48 mV",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1111 51 mV",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_KillThreshold), GTK_TREE_MODEL(store));


  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_KillThreshold), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_KillThreshold ), cell, "text", 0, NULL );

  drawGain();
}

void drawTest(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_TEST_SIGNAL);
  reg = reg_now & 0x3;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_TestBars), reg);
  reg = (reg_now & 0x18) >> 3;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_TestTiming), reg);

  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_TestADC, reg_now & 0x80);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_TestPattern, reg_now & 0x20);
}

void setupTest(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"00 = Black screen (default)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"01 = Blue screen",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"10 = 75% color bars",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"11 = 100% color bars",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_TestBars), GTK_TREE_MODEL(store));

  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_TestBars), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_TestBars ), cell, "text", 0, NULL );

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"00, test pattern with 50Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"01 test pattern with last used timing ",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"10, test pattern with 60Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"11 test pattern with last used timing ",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_TestTiming), GTK_TREE_MODEL(store));

  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_TestTiming), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_TestTiming ), cell, "text", 0, NULL );

  drawGain();
}


void drawClock(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_CLOCK);
  reg = (reg_now & 0x40) >> 6;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_ClockClip), reg);
  reg = (reg_now & 0x18) >> 3;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_ClockOsz), reg);

  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ClockInvLLC, reg_now & 0x20);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ClockSync, reg_now & 0x04);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ClockDisableDx, reg_now & 0x02);
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_ClockDisableLLC, reg_now & 0x01);
}

void setupClock(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"00 = 27 MHz Crystal",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"01 = 27 MHz Oszillator",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"10 = N/A",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"11 = 54 MHz Oszillator",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_ClockOsz), GTK_TREE_MODEL(store));


  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_ClockOsz), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_ClockOsz ), cell, "text", 0, NULL );

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"0 = Clip ITU Y, CbCr range 5–1019 (default) ",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"1 = Clip ITU Y, CbCr range 64–960 ",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_ClockClip), GTK_TREE_MODEL(store));


  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_ClockClip), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_ClockClip ), cell, "text", 0, NULL );

  drawClock();
}

void drawPLL(void)
{
  unsigned reg,reg_now;

  reg_now = read_reg_max9526(MAX9526_PLL_CTRL);
  reg = (reg_now >> 4) & 0x3;
  gtk_combo_box_set_active(GTK_COMBO_BOX(data->cob_PLLClock), reg);

  reg = reg_now & 0x7;
  gtk_combo_box_set_active(GTK_COMBO_BOX( data->cob_PLLTracking), reg);

  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_PLL, reg_now & 0x08);
}

void setupPLL(void)
{
  GtkTreeIter iter;
  GtkListStore *store;
  GtkCellRenderer *cell;

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"00 = Async / line-locked auto (default)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"01 = Async / line-locked auto (default)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"10 = PLL is forced to line-locked mode",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"11 = PLL is forced to async mode",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX(data->cob_PLLClock), GTK_TREE_MODEL(store));

  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(data->cob_PLLClock), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( data->cob_PLLClock ), cell, "text", 0, NULL );

  store = gtk_list_store_new(1,G_TYPE_STRING);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"000 = 180Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"001 = 250Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"010 = 375Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"011 = 500Hz (default)",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"100 = 750Hz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"101 = 1kHz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"110 = 1.5kHz",-1);
  gtk_list_store_append(store,&iter);
  gtk_list_store_set(store,&iter,0,"111 = 2kHz",-1);

  gtk_combo_box_set_model(GTK_COMBO_BOX( data->cob_PLLTracking), GTK_TREE_MODEL(store));

  cell = gtk_cell_renderer_text_new();

  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT( data->cob_PLLTracking), cell, TRUE);
  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT(  data->cob_PLLTracking ), cell, "text", 0, NULL );

  drawPLL();
}

void drawMisc (void)
{
  unsigned reg, reg_now;

  reg = read_reg_max9526(MAX9526_MISCELLANEOUS);

printf("drawMisc(void) 1\n");

  reg_now = reg & 0xf;
  gtk_range_set_value ( (GtkRange*) data->vs_MiscSSLICE, reg_now );
  gtk_toggle_button_set_active((GtkToggleButton *)data->cb_MiscAGC, reg & 0x20);

printf("drawMisc(void) 2\n");

}
void setupMisc(void)
{
  GtkAdjustment *adjustment;
  adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, 0.0, 15.0, 1.0, 4.0, 0.0);
  
printf("setupMisc(void) 1\n");

  gtk_range_set_adjustment( (GtkRange*)data->vs_MiscSSLICE, adjustment );
printf("setupMisc(void) 2\n");

  gtk_range_set_inverted ((GtkRange*) data->vs_MiscSSLICE, TRUE);
  
printf("setupMisc(void) 2\n");

}

int main (int argc, char **argv)
{
  GtkBuilder *builder;
  GError *error = NULL;

  /* Init I2C */
  init_max9526();
  
  /* Init GTK+ */
  gtk_init (&argc, &argv);

  /* Create new GtkBuilder object */
  builder = gtk_builder_new ();
  /* Load UI from file. If error occurs, report it and quit application.
   * Replace "tut.glade" with your saved project. */
  if (!gtk_builder_add_from_file (builder, UI_FILE, &error))
    {
      g_warning ("%s", error->message);
      g_free (error);
      return (1);
    }

  /* Allocate data structure */
  data = g_slice_new( ChData );

  /* Get objects from UI */
#define GW( name ) CH_GET_WIDGET( builder, name, data )
  GW( main_window );
  GW( cb_VID1 );
  GW( cb_VID2 );
  GW( cb_CTHR );
  GW( cb_ADCOVR );
  GW( cb_HLOCK );
  GW( cb_NONSTD );
  GW( cb_LSTLCK );
  
  GW( cb_L525 );
  GW( cb_ACP );
  
  GW( cb_IVID1 );
  GW( cb_IVID2 );
  GW( cb_ICTHR );
  GW( cb_IADCOVR );
  GW( cb_IHLOCK );
  GW( cb_INONSTD );
  GW( cb_ILSTLCK );

  GW( cb_IL525 );
  GW( cb_IACP );

  GW( cob_VideoStandard);
  GW( cb_AUTOD );
  GW( cb_SHDN );
  GW( cb_SLEEP );
  
  GW( vs_Contrast );
  GW( lb_LumaGain );
  GW( vs_Brightness );
  GW( lb_IRE );
  GW( vs_Hue );
  GW( lb_Degrees );
  GW( vs_Saturation );
  GW( lb_ChromaGain );
  
  GW( cob_VideoInputSrc );
  GW( cob_DCRestore );
  GW( cb_DigitalClampDis );
  
  GW( cob_GainADC );
  GW( cb_GainCoAGC );
  GW( cb_GainChAGC );
  GW( cb_GainADCAGC );

  GW( cob_KillThreshold );
  GW( cb_KillColour );
  GW( cb_KillBW );

  GW( cob_TestBars );
  GW( cob_TestTiming );
  GW( cb_TestADC );
  GW( cb_TestPattern );

  GW( cob_ClockClip );
  GW( cob_ClockOsz );
  GW( cb_ClockInvLLC );
  GW( cb_ClockDisableDx );
  GW( cb_ClockDisableLLC );
  GW( cb_ClockSync );

  GW( cob_PLLClock );
  GW( cob_PLLTracking );
  GW( cb_PLL );

  GW( cb_MiscAGC );

  GW( vs_MiscSSLICE );
#undef GW

  /* Connect signals */
  gtk_builder_connect_signals (builder, data );

  /* Destroy builder, since we don't need it anymore */
  g_object_unref (G_OBJECT (builder));
  
  setupVideoStandard();
  printf("setupVideoStandard();\n");
  setupVideoInputSelect();
  printf("setupVideoInputSelect();\n");
  setupSliders();
  printf("setupSliders();\n");
  setupGain();
  printf("setupGain();\n");
  setupKill();
  printf("setupKill();\n");
  setupTest();
  printf("setupTest();\n");
  setupClock();
  printf("setupTest();\n");
  setupPLL();
  printf("setupPLL();\n");
  setupMisc();
  printf("setupMisc();\n");
  refresh ();
  printf("refresh();\n");

  /* Show window. All other widgets are automatically shown by GtkBuilder */
  gtk_widget_show ( data->main_window );
  
  
  is_initialized = TRUE;

  
  /* Start main loop */
  gtk_main ();
  
  close_max9526();

  return (0);
}
